## $Id: test_browser.py 12840 2015-04-01 08:35:16Z henrik $
##
## Copyright (C) 2015 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""
Extends the doctests in browser.txt and datacenter.txt.
"""
import shutil
import tempfile
import os
from zc.async.testing import wait_for_result
from zope.component import createObject, getUtility
from zope.component.hooks import setSite, clearSite
from zope.security.interfaces import Unauthorized
from zope.securitypolicy.interfaces import IPrincipalRoleManager
from zope.testbrowser.testing import Browser
from waeup.ikoba.testing import FunctionalLayer, FunctionalTestCase
from waeup.ikoba.app import Company
from waeup.ikoba.interfaces import IJobManager
from waeup.ikoba.tests.test_async import FunctionalAsyncTestCase


class CompanySetup(FunctionalTestCase):
    # A test case that only contains a setup and teardown

    layer = FunctionalLayer

    def setUp(self):
        super(CompanySetup, self).setUp()

        # Setup a sample site for each test
        app = Company()
        self.dc_root = tempfile.mkdtemp()
        app['datacenter'].setStoragePath(self.dc_root)

        # Prepopulate the ZODB...
        self.getRootFolder()['app'] = app
        # we add the site immediately after creation to the
        # ZODB. Catalogs and other local utilities are not setup
        # before that step.
        self.app = self.getRootFolder()['app']
        # Set site here. Some of the following setup code might need
        # to access grok.getSite() and should get our new app then
        setSite(app)

        # Prepare test browser
        self.browser = Browser()
        self.browser.handleErrors = False

        self.datacenter_path = 'http://localhost/app/datacenter'

    def tearDown(self):
        super(CompanySetup, self).tearDown()
        clearSite()
        shutil.rmtree(self.dc_root)


class DataCenterUITests(CompanySetup, FunctionalAsyncTestCase):
    # Tests for DataCenter class views and pages

    layer = FunctionalLayer

    def wait_for_export_job_completed(self):
        # helper function waiting until the current export job is completed
        manager = getUtility(IJobManager)
        job_id = self.app['datacenter'].running_exports[0][0]
        job = manager.get(job_id)
        wait_for_result(job)
        return job_id

    def cleanup_run_job(self):
        # helper to remove any result file of an export
        job_id = self.wait_for_export_job_completed()
        manager = getUtility(IJobManager)
        job = manager.get(job_id)
        if os.path.exists(job.result):
            shutil.rmtree(os.path.dirname(job.result))

    def test_user_data_import_permission(self):
        upload_path = 'http://localhost/app/datacenter/upload'
        login_path = 'http://localhost/app/login'
        # Create  officer
        self.app['users'].addUser('officer', 'officersecret')
        self.app['users']['officer'].email = 'mrofficer@foo.ng'
        self.app['users']['officer'].title = 'Carlo Pitter'
        # Assign role
        prm = IPrincipalRoleManager(self.app)
        prm.assignRoleToPrincipal('waeup.ImportManager', 'officer')
        # Login
        self.browser.open(login_path)
        self.browser.getControl(name="form.login").value = 'officer'
        self.browser.getControl(name="form.password").value = 'officersecret'
        self.browser.getControl("Login").click()
        self.browser.open(upload_path)
        self.assertFalse('User Processor' in self.browser.contents)
        # Assign role which is necessary to import user data
        prm.assignRoleToPrincipal('waeup.UsersManager', 'officer')
        self.browser.open(upload_path)
        self.assertTrue('User Processor' in self.browser.contents)
        return

    def test_export_disabled(self):
        self.app['configuration'].export_disabled_message = u'Export forbidden'
        self.browser.addHeader('Authorization', 'Basic mgr:mgrpw')
        self.browser.open(self.datacenter_path)
        self.assertEqual(self.browser.headers['Status'], '200 Ok')
        self.assertEqual(self.browser.url, self.datacenter_path)
        self.browser.getLink("Export data").click()
        self.browser.getControl(name="exporter").value = ['products']
        self.browser.getControl("Create CSV file").click()
        self.assertEqual(self.browser.headers['Status'], '200 Ok')
        self.assertTrue('div class="alert alert-danger">Export forbidden</div>'
                         in self.browser.contents)
        return

    def test_export_start(self):
        # we can trigger export file creation
        self.browser.addHeader('Authorization', 'Basic mgr:mgrpw')
        self.browser.open(self.datacenter_path)
        self.assertEqual(self.browser.headers['Status'], '200 Ok')
        self.assertEqual(self.browser.url, self.datacenter_path)
        self.browser.getLink("Export data").click()
        self.browser.getControl(name="exporter").value = ['products']
        self.browser.getControl("Create CSV file").click()
        # we can trigger export file creation only once
        self.browser.getControl(name="exporter").value = ['pdfdocuments']
        self.browser.getControl("Create CSV file").click()
        self.assertTrue('Sorry, another export job is running.'
                         in self.browser.contents)
        self.wait_for_export_job_completed()
        self.assertEqual(len(self.app['datacenter'].running_exports), 1)
        self.browser.getControl(name="exporter").value = ['pdfdocuments']
        self.browser.getControl("Create CSV file").click()
        self.assertEqual(self.browser.headers['Status'], '200 Ok')
        self.assertEqual(len(self.app['datacenter'].running_exports), 2)
        # The second job is running
        self.assertTrue('<img src="/static/img/actionicon_reload.png" />'
                         in self.browser.contents)
        self.cleanup_run_job() # make sure to remove temp dirs
        # Also the second job must be cleaned up
        manager = getUtility(IJobManager)
        job_id = self.app['datacenter'].running_exports[1][0]
        job = manager.get(job_id)
        wait_for_result(job)
        if os.path.exists(job.result):
            shutil.rmtree(os.path.dirname(job.result))
        return

